/*************************************************************************
* (C) 2010 AITNET ltd - Sofia/Bulgaria - <misho@aitbg.com>
*  by Michael Pounov <misho@openbsd-bg.org>
*
* $Author: misho $
* $Id: aitsync.h,v 1.6 2014/02/04 16:58:17 misho Exp $
*
**************************************************************************
The ELWIX and AITNET software is distributed under the following
terms:

All of the documentation and software included in the ELWIX and AITNET
Releases is copyrighted by ELWIX - Sofia/Bulgaria <info@elwix.org>

Copyright 2004 - 2014
	by Michael Pounov <misho@elwix.org>.  All rights reserved.

Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions
are met:
1. Redistributions of source code must retain the above copyright
   notice, this list of conditions and the following disclaimer.
2. Redistributions in binary form must reproduce the above copyright
   notice, this list of conditions and the following disclaimer in the
   documentation and/or other materials provided with the distribution.
3. All advertising materials mentioning features or use of this software
   must display the following acknowledgement:
This product includes software developed by Michael Pounov <misho@elwix.org>
ELWIX - Embedded LightWeight unIX and its contributors.
4. Neither the name of AITNET nor the names of its contributors
   may be used to endorse or promote products derived from this software
   without specific prior written permission.

THIS SOFTWARE IS PROVIDED BY AITNET AND CONTRIBUTORS ``AS IS'' AND
ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
ARE DISCLAIMED.  IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE LIABLE
FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
SUCH DAMAGE.
*/
#ifndef __AITSYNC_H
#define __AITSYNC_H


// sync_GetErrno() Get error code of last operation
int sync_GetErrno();
// sync_GetError() Get error text of last operation
const char *sync_GetError();

/*
 * syncSignature() - Calculate and create signature for diff
 *
 * @csInput = Input target file name for calculating check sums
 * @csSig = Output Signature file name
 * @compress = 2 compress signatures output, 0 not compressed
 * return: -1 error, 0 ok
 */
int syncSignature(const char *csInput, const char *csSig, int compress);

/*
 * syncDelta() - Create Delta patch file
 *
 * @csInput = Input original source file name for make delta patch file
 * @csSig = Input target Signature file name
 * @csDelta = Output Delta patch file name
 * @compress = 3 everything compress, 2 compressed signatures, 1 compress delta output, 0 not compressed
 * return: -1 error, 0 ok, create delta patch, 1 ok, no differences and not create patch
 */
int syncDelta(const char *csInput, const char *csSig, const char *csDelta, int compress);

/*
 * syncPatch() - Apply delta patch file to target
 *
 * @csInput = Input target file name for patch
 * @csDelta = Input Delta patch file name
 * @csPatch = After applied patch create new alternate target file, if != NULL
 * @compress = 1 compress delta input, 0 not compressed
 * return: -1 error, 0 ok, create delta patch, 1 ok, no differences and not create patch
 */
int syncPatch(const char *csInput, const char *csDelta, const char *csPatch, int compress);


struct tagDirName {
	u_char	ch;
	u_short tag;
	u_int	hash;
	char	name[BUFSIZ];
	char	extra[BUFSIZ];
};

/*
 * sync_dircmp() - Compare directories
 *
 * @csDir1 = Directory 1
 * @csDir2 = Directory 2
 * return: -1 error, 0 is equal or 1 different
 */
int sync_dircmp(const char *csDir1, const char *csDir2);
/*
 * sync_dirChkSum() - Calculate checksum of directory
 *
 * @csDir = Directory
 * @md = Message digest allocated memory, must be e_free() after use!
 * return: -1 error or !=-1 ok
 */
int sync_dirChkSum(const char *csDir, u_char **md);
/*
 * sync_dircmpList() - Compare directories or directory and file list
 *
 * @csDir1 = Directory 1
 * @csDir2 = Directory 2 or File list, if "-" get input from console
 * @lm = Long mode options, !=NULL long output
 * @list = Output diff list, after use must be e_free()!
 * return: -1 error, 0 is equal or >0 count of returned list items
 */
int sync_dircmpList(const char *csDir1, const char *csDir2, int lm, struct tagDirName **list);


#endif
