/*************************************************************************
* (C) 2013 AITNET ltd - Sofia/Bulgaria - <misho@aitnet.org>
*  by Michael Pounov <misho@elwix.org>
*
* $Author: misho $
* $Id: acrc.h,v 1.6 2017/12/08 00:07:48 misho Exp $
*
**************************************************************************
The ELWIX and AITNET software is distributed under the following
terms:

All of the documentation and software included in the ELWIX and AITNET
Releases is copyrighted by ELWIX - Sofia/Bulgaria <info@elwix.org>

Copyright 2004 - 2017
	by Michael Pounov <misho@elwix.org>.  All rights reserved.

Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions
are met:
1. Redistributions of source code must retain the above copyright
   notice, this list of conditions and the following disclaimer.
2. Redistributions in binary form must reproduce the above copyright
   notice, this list of conditions and the following disclaimer in the
   documentation and/or other materials provided with the distribution.
3. All advertising materials mentioning features or use of this software
   must display the following acknowledgement:
This product includes software developed by Michael Pounov <misho@elwix.org>
ELWIX - Embedded LightWeight unIX and its contributors.
4. Neither the name of AITNET nor the names of its contributors
   may be used to endorse or promote products derived from this software
   without specific prior written permission.

THIS SOFTWARE IS PROVIDED BY AITNET AND CONTRIBUTORS ``AS IS'' AND
ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
ARE DISCLAIMED.  IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE LIABLE
FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
SUCH DAMAGE.
*/
#ifndef __ACRC_H
#define __ACRC_H


#define REVOPTS_REVERTBYTE	1
#define REVOPTS_REVERTCRC	2


struct tagCRCPoly {
	u_char	poly_bits;
	u_int	poly_num;
	char	poly_name[19];
};	/* size 24bytes */
typedef struct tagCRCPoly crcPoly_t;


/*
 * crcReflect() - Reflect all bits of number 
 *
 * @crcNum = Number for reflection
 * @crcBits = Number width bits 
 * return: -1 error, !=-1 reflecting number
 */
unsigned int crcReflect(unsigned int crcNum, unsigned char crcBits);
/*
 * crcCalc() - Generic CRC calculation function for many sub variants of CRC algorithms
 *
 * @psBuf = Data for calculation
 * @bufLen = Length of data
 * @crcBits = CRC algorithm bits (1, 4, 5, 6, 7, 8, 10, 11, 12, 15, 16, 24, 30, 32)
 * @RevOpts = Options for computation (REVOPTS_REVERTBYTE, REVOPTS_REVERTCRC)
 * @initCRC = Initial CRC value
 * @xorCRC = Last xor CRC value
 * return: -1 error, !=-1 CRC checksum
 */
unsigned int crcCalc(unsigned char * __restrict psBuf, unsigned int bufLen, 
		unsigned char crcBits, unsigned char RevOpts, 
		unsigned int initCRC, unsigned int xorCRC);

/*
 * crcIP() - Checksum in IP communication
 *
 * @buf = Data for calculation
 * @bufLen = Length of data
 * return: -1 error, !=-1 Checksum
 */
unsigned short crcIP(unsigned char * __restrict buf, int bufLen);
/*
 * crcTCP() - Checksum for TCP v4 communication
 *
 * @buf = Data for calculation
 * @bufLen = Length of data
 * @th = TCP header
 * return: -1 error, !=-1 Checksum
 */
unsigned short crcTCP(struct in_addr src, struct in_addr dst, 
		unsigned char * __restrict th);
/*
 * crcUDP() - Checksum for UDP v4 communication
 *
 * @buf = Data for calculation
 * @bufLen = Length of data
 * @uh = UDP header
 * return: -1 error, !=-1 Checksum
 */
unsigned short crcUDP(struct in_addr src, struct in_addr dst, 
		unsigned char * __restrict uh);

/*
 * crcFletcher16() - Fletcher-16 Checksum computing
 *
 * @nBuf = Data for calculation
 * @bufLen = Length of data
 * return: -1 error, !=-1 Checksum
 */
unsigned short crcFletcher16(unsigned short * __restrict nBuf, int bufLen);
/*
 * crcFletcher() - Fletcher-32 Checksum computing
 *
 * @nBuf = Data for calculation
 * @bufLen = Length of data
 * return: -1 error, !=-1 Checksum
 */
unsigned int crcFletcher(unsigned short * __restrict nBuf, int bufLen);
/*
 * crcAdler() - crcAdler-32 Checksum computing
 *
 * @psBuf = Data for calculation
 * @bufLen = Length of data
 * return: -1 error, !=-1 Checksum
 */
unsigned int crcAdler(unsigned char * __restrict psBuf, int bufLen);

/*
 * crcEther() - Checksum in Ethernet communication
 *
 * @psBuf = Data for calculation
 * @bufLen = Length of data
 * return: -1 error, !=-1 Checksum
 */
#define crcEther(psBuf, bufLen) crcCalc((psBuf), (bufLen), 32, 3, 0xFFFFFFFF, 0xFFFFFFFF)

/*
 * crc_32() - CRC32 calculation from table
 *
 * @crc = Initial crc value
 * @buf = Data for calculation
 * @len = Length of data
 * return: calculated CRC32
 */
unsigned int crc_32(unsigned int crc, const unsigned char * __restrict buf, unsigned int len);

/*
 * crcPelco() - Calculate Pelco D/P CRC
 *
 * @ver = Pelco protocol version (Dd | Pp)
 * @pkt = Packet for calculate crc
 * return: crc for packet, if is 0 check and crc_GetErrno() == 1 
 	Pelco protocol not supported
 */
unsigned char crcPelco(unsigned char ver, unsigned char *pkt);


/*
 * hash_varchar() - Compute index hash by variable length string
 *
 * @csStr = Input data buffer
 * @nStrLen = Length of data buffer
 * @nVer = Version of algorythm; 0 - original, 1 - AITNET variant
 * return: Hash value
*/
unsigned int hash_varchar(const char *csStr, int nStrLen, int nVer);
/*
 * hash_bernstein() - Compute index hash by Bernstein
 *
 * @csStr = Input data buffer
 * @nStrLen = Length of data buffer
 * @nVer = Version of algorythm; 0 - Bernstein, 1 - DJBX33A variant
 * return: Hash value
*/
unsigned int hash_bernstein(const char *csStr, int nStrLen, int nVer);
/*
 * hash_jenkins() - Compute index hash by Jenkins (one-at-a-time)
 *
 * @csStr = Input data buffer
 * @nStrLen = Length of data buffer
 * return: Hash value
*/
unsigned int hash_jenkins(const char *csStr, int nStrLen);
/*
 * hash_jenkins32() - Fast Jenkins hash function
 *
 * @buf = Input buffer
 * @len = Length of buffer
 * @prevhash = Previous hash, if participate in continuing hash
 * return: Hash value
 */
unsigned int hash_jenkins32(const unsigned int *buf, int len, unsigned int prevhash);
/*
 * hash_reddragon() - Compute index hash by Red Dragon
 *
 * @csStr = Input data buffer
 * @nStrLen = Length of data buffer
 * return: Hash value
*/
unsigned int hash_reddragon(const char *csStr, int nStrLen);
/*
 * hash_fnv1() - Compute index hash by FNV-1
 *
 * @csStr = Input data buffer
 * @nStrLen = Length of data buffer
 * @nVer = Version of algorythm; 0 - FNV-1, 1 - FNV-1a (best avalanche)
 * return: Hash value
*/
unsigned int hash_fnv1(const char *csStr, int nStrLen, int nVer);

/*
 * hash_fnv() - Compute index hash by FNV-1a
 *
 * @csStr = Input data buffer
 * @nStrLen = Length of data buffer
 * return: Hash value
*/
#define hash_fnv(str, len)	hash_fnv1((str), (len), 1)


#endif
