/*************************************************************************
* (C) 2007 AITNET ltd - Sofia/Bulgaria - <misho@aitnet.org>
*  by Michael Pounov <misho@elwix.org>
*
* $Author: misho $
* $Id: apatricia.h,v 1.4 2015/06/25 17:53:49 misho Exp $
*
**************************************************************************
The ELWIX and AITNET software is distributed under the following
terms:

All of the documentation and software included in the ELWIX and AITNET
Releases is copyrighted by ELWIX - Sofia/Bulgaria <info@elwix.org>

Copyright 2004 - 2015
	by Michael Pounov <misho@elwix.org>.  All rights reserved.

Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions
are met:
1. Redistributions of source code must retain the above copyright
   notice, this list of conditions and the following disclaimer.
2. Redistributions in binary form must reproduce the above copyright
   notice, this list of conditions and the following disclaimer in the
   documentation and/or other materials provided with the distribution.
3. All advertising materials mentioning features or use of this software
   must display the following acknowledgement:
This product includes software developed by Michael Pounov <misho@elwix.org>
ELWIX - Embedded LightWeight unIX and its contributors.
4. Neither the name of AITNET nor the names of its contributors
   may be used to endorse or promote products derived from this software
   without specific prior written permission.

THIS SOFTWARE IS PROVIDED BY AITNET AND CONTRIBUTORS ``AS IS'' AND
ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
ARE DISCLAIMED.  IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE LIABLE
FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
SUCH DAMAGE.
*/
/*
 * This product includes software developed by the University of Michigan,
 * Merit Network, Inc., and their contributors. 
 *
 * This file had been called "radix.h" in the MRT sources.
 *
 * I renamed it to "apatricia.h" since it's not an implementation of a general
 * radix trie.  Also, pulled in various requirements from "mrt.h" and added
 * some other things it could be used as a standalone API.
 *
 * Added and patched existing code by AITNET - Michael Pounov <misho@aitbg.com>
 */
#ifndef __APATRICIA_H
#define __APATRICIA_H

#include <netinet/in.h> /* for struct in_addr */
#include <sys/socket.h> /* for AF_INET */

/* typedef unsigned int u_int; in most cases is NULL for functions parameter */
typedef void (*void_fn_t)();

/* { from mrt.h */

typedef struct _prefix4_t {
    u_short		family;		/* AF_INET | AF_INET6 */
    u_short		bitlen;		/* same as mask? */
    int			ref_count;	/* reference count */
    struct in_addr	sin;
} prefix4_t;

typedef struct _prefix_t {
    u_short		family;		/* AF_INET | AF_INET6 */
    u_short		bitlen;		/* same as mask? */
    int			ref_count;	/* reference count */
    union {
		struct in_addr	sin;
#ifdef HAVE_IPV6
		struct in6_addr	sin6;
#endif /* IPV6 */
    } add;
} prefix_t;

/* } */

typedef struct _patricia_node_t {
   u_int			bit;		/* flag if this node used */
   prefix_t			*prefix;	/* who we are in patricia tree */
   struct _patricia_node_t	*l, *r;		/* left and right children */
   struct _patricia_node_t	*parent;	/* may be used */
   void				*data;		/* pointer to data */
   void				*user1;		/* pointer to usr data (ex. route flap info) */
} patricia_node_t;

typedef struct _patricia_tree_t {
   patricia_node_t	*head;
   u_int		maxbits;		/* for IP, 32 bit addresses */
   int			num_active_node;	/* for debug purpose */
} patricia_tree_t;


patricia_tree_t *New_Patricia(int maxbits);
void Destroy_Patricia(patricia_tree_t *patricia, void_fn_t func);
void Clear_Patricia(patricia_tree_t *patricia, void_fn_t func);
prefix_t *Ref_Prefix(prefix_t *prefix);
void Deref_Prefix(prefix_t * prefix);

patricia_node_t *patricia_lookup(patricia_tree_t *patricia, prefix_t *prefix);
void patricia_remove(patricia_tree_t *patricia, patricia_node_t *node);
void patricia_process(patricia_tree_t *patricia, void_fn_t func);

patricia_node_t *patricia_search_exact(patricia_tree_t *patricia, prefix_t *prefix);
patricia_node_t *patricia_search_best(patricia_tree_t *patricia, prefix_t *prefix);
patricia_node_t *patricia_search_best2(patricia_tree_t *patricia, prefix_t *prefix, int inclusive);

/* { from demo.c */
char *prefix_toa(prefix_t *prefix);
prefix_t *ascii2prefix(int family, char *string);

#define addroute make_and_lookup
patricia_node_t *make_and_lookup(patricia_tree_t *tree, char *string);
#define delroute lookup_then_remove
void lookup_then_remove(patricia_tree_t *tree, char *string);
patricia_node_t *try_search_exact(patricia_tree_t *tree, char *string);
patricia_node_t *try_search_best(patricia_tree_t *tree, char *string);
/* } */

/* { from defs.h */
#define prefix_touchar(prefix)	((u_char *) &(prefix)->add.sin)

#define Delete free

#define MAXLINE			1024
#define BIT_TEST(f, b)		((f) & (b))
/* } */

#define PATRICIA_MAXBITS	128
#define PATRICIA_NBIT(x)	(0x80 >> ((x) & 0x7f))
#define PATRICIA_NBYTE(x)	((x) >> 3)

#define PATRICIA_DATA_GET(node, type)	(type *)((node)->data)
#define PATRICIA_DATA_SET(node, value)	((node)->data = (void *)(value))

#define PATRICIA_WALK(Xhead, Xnode) \
    do { \
        patricia_node_t *Xstack[PATRICIA_MAXBITS + 1]; \
        patricia_node_t **Xsp = Xstack; \
        patricia_node_t *Xrn = (Xhead); \
        while ((Xnode = Xrn)) { \
            if (Xnode->prefix)

#define PATRICIA_WALK_ALL(Xhead, Xnode) \
do { \
        patricia_node_t *Xstack[PATRICIA_MAXBITS + 1]; \
        patricia_node_t **Xsp = Xstack; \
        patricia_node_t *Xrn = (Xhead); \
        while ((Xnode = Xrn)) { \
	    if (42)

#define PATRICIA_WALK_BREAK { \
	    if (Xsp != Xstack) { \
		Xrn = *(--Xsp); \
	     } else { \
		Xrn = (patricia_node_t *) 0; \
	    } \
	    continue; }

#define PATRICIA_WALK_END \
            if (Xrn->l) { \
                if (Xrn->r) { \
                    *Xsp++ = Xrn->r; \
                } \
                Xrn = Xrn->l; \
            } else if (Xrn->r) { \
                Xrn = Xrn->r; \
            } else if (Xsp != Xstack) { \
                Xrn = *(--Xsp); \
            } else { \
                Xrn = (patricia_node_t *) 0; \
            } \
        } \
} while (0)

#endif /* __APATRICIA_H */
