/*************************************************************************
* (C) 2013 AITNET ltd - Sofia/Bulgaria - <misho@aitnet.org>
*  by Michael Pounov <misho@elwix.org>
*
* $Author: misho $
* $Id: apio.h,v 1.4 2015/06/25 17:53:49 misho Exp $
*
**************************************************************************
The ELWIX and AITNET software is distributed under the following
terms:

All of the documentation and software included in the ELWIX and AITNET
Releases is copyrighted by ELWIX - Sofia/Bulgaria <info@elwix.org>

Copyright 2004 - 2015
	by Michael Pounov <misho@elwix.org>.  All rights reserved.

Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions
are met:
1. Redistributions of source code must retain the above copyright
   notice, this list of conditions and the following disclaimer.
2. Redistributions in binary form must reproduce the above copyright
   notice, this list of conditions and the following disclaimer in the
   documentation and/or other materials provided with the distribution.
3. All advertising materials mentioning features or use of this software
   must display the following acknowledgement:
This product includes software developed by Michael Pounov <misho@elwix.org>
ELWIX - Embedded LightWeight unIX and its contributors.
4. Neither the name of AITNET nor the names of its contributors
   may be used to endorse or promote products derived from this software
   without specific prior written permission.

THIS SOFTWARE IS PROVIDED BY AITNET AND CONTRIBUTORS ``AS IS'' AND
ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
ARE DISCLAIMED.  IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE LIABLE
FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
SUCH DAMAGE.
*/
#ifndef __APIO_H
#define __APIO_H


struct tagPIOPID {
	union {
		FILE		*fp;
		int		fd;
	} f;
	pid_t			pid;
	int			stat;

	SLIST_ENTRY(tagPIOPID)	next;
};
typedef SLIST_HEAD(, tagPIOPID) pio_pid_t;


extern pio_pid_t pio_pidlist;


/*
 * e_popen() - ELWIX replacement of standard popen
 *
 * @command = command
 * @type = type
 * @ppid = return pid of child program
 * return: NULL error or !=NULL open program
 */
#ifdef POPEN_STREAM
FILE *e_popen(const char *command, const char *type, pid_t *ppid);
#else
int e_popen(const char *command, const char *type, pid_t *ppid);
#endif
/*
 * e_pclose() - ELWIX replacement of standard pclose
 *
 * @iop = popen handle
 * return: -1 error or !=-1 pid status
 */
#ifdef POPEN_STREAM
int e_pclose(FILE *iop);
#else
int e_pclose(int iop);
#endif

/*
 * pio_pgetpid() - Get tagPIOPID structure from file handle
 *
 * @iop = popen handle
 * return: NULL error or !=NULL tagPIOPID structure
 */
#ifdef POPEN_STREAM
struct tagPIOPID *pio_pgetpid(FILE *iop);
#else
struct tagPIOPID *pio_pgetpid(int iop);
#endif
/*
 * pio_pchkpid() - Check exit status of child programs
 *
 * @pids = return tagPIOPID structures of exited programs, 
 * 		if !=NULL must call array_Destroy()
 * return: -1 error or >-1 exited programs
 */
int pio_pchkpid(array_t ** __restrict pids);


#endif
