/*************************************************************************
* (C) 2013 AITNET ltd - Sofia/Bulgaria - <misho@aitnet.org>
*  by Michael Pounov <misho@elwix.org>
*
* $Author: misho $
* $Id: asarr.h,v 1.4 2015/06/25 17:53:49 misho Exp $
*
**************************************************************************
The ELWIX and AITNET software is distributed under the following
terms:

All of the documentation and software included in the ELWIX and AITNET
Releases is copyrighted by ELWIX - Sofia/Bulgaria <info@elwix.org>

Copyright 2004 - 2015
	by Michael Pounov <misho@elwix.org>.  All rights reserved.

Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions
are met:
1. Redistributions of source code must retain the above copyright
   notice, this list of conditions and the following disclaimer.
2. Redistributions in binary form must reproduce the above copyright
   notice, this list of conditions and the following disclaimer in the
   documentation and/or other materials provided with the distribution.
3. All advertising materials mentioning features or use of this software
   must display the following acknowledgement:
This product includes software developed by Michael Pounov <misho@elwix.org>
ELWIX - Embedded LightWeight unIX and its contributors.
4. Neither the name of AITNET nor the names of its contributors
   may be used to endorse or promote products derived from this software
   without specific prior written permission.

THIS SOFTWARE IS PROVIDED BY AITNET AND CONTRIBUTORS ``AS IS'' AND
ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
ARE DISCLAIMED.  IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE LIABLE
FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
SUCH DAMAGE.
*/
#ifndef __ASARR_H
#define __ASARR_H


typedef void ** sarr_seg_t;
typedef struct _tagSplitArray {
	int		sarr_num;
	int		sarr_seg;
	int		sarr_siz;
	sarr_seg_t	*sarr_data;
} sarr_t;


/*
 * sarr_Init() - Create and initialize dynamic split-order array
 *
 * @numItems = Number of Items
 * @segLen = Length of segment
 * return: NULL error, != NULL allocated memory for array
 */
sarr_t *sarr_Init(int numItems, int segLen);
/*
 * sarr_Destroy() - Free all data in dynamic split-order array and Destroy array
 *
 * @parr = Array
 * return: none
 */
void sarr_Destroy(sarr_t ** __restrict parr);

/*
 * sarr_Grow() - Grow/Shrink dynamic split-order array, Use with care when it shrink!!!
 *
 * @arr = Array
 * @newNumItems = Number of Items
 * return: -1 error, 0 ok
 */
int sarr_Grow(sarr_t * __restrict arr, int newNumItems);
/*
 * sarr_Vacuum() - Vacuum dynamic split-order array, empty segments will be freed
 *
 * @arr = Array
 * return: -1 error, >-1 freed segments
 */
int sarr_Vacuum(sarr_t * __restrict arr);
#define sarr_Size(_arr)		((_arr) ? (_arr)->sarr_num : 0)
#define sarr_Seg(_arr)		(assert((_arr)), (_arr)->sarr_seg)
/*
 * sarr_Copy() Copy source split array to destination split array
 *
 * @dest = Destination split array, after use free with sarr_Destroy()
 * @src = Source split array
 * return: -1 error; >0 count of destination split array
 */
int sarr_Copy(sarr_t ** __restrict dest, sarr_t * __restrict src);

/*
 * sarr_Get() - Get element from dynamic split-order array
 *
 * @arr = Array
 * @idx = Index (warning 1st element is at position 1)
 * return: NULL not found, !=NULL element
 */
void *sarr_Get(sarr_t * __restrict arr, unsigned int idx);
/*
 * sarr_Get2() - Always get element from dynamic split-order array
 *	Function automatic grow array. Good use for Hash tables! 
 *
 * @arr = Array
 * @idx = Index (warning 1st element is at position 1)
 * return: NULL not found, !=NULL element
 */
void *sarr_Get2(sarr_t * __restrict arr, unsigned int idx);
/*
 * sarr_Set() - Set element to dynamic split-order array
 *
 * @arr = Array
 * @idx = Index (warning 1st element is at position 1)
 * @data = Value
 * return: NULL error or empty, !=NULL old value in element
 */
void *sarr_Set(sarr_t * __restrict arr, unsigned int idx, void *data);
#define sarr_Del(_arr, _idx)		sarr_Set((_arr), _idx, NULL)
#define sarr(_arr, _idx, _type)		((_type) sarr_Get((_arr), _idx))

/*
 * sarr_sarr_2array() - Convert from split-order array to dynamic array
 *
 * @sa = split array
 * @sarrFree = after convert split array !=0 will be destroyed sarray
 * return: NULL error or != NULL new array
 */
array_t *sarr_sarr2array(sarr_t ** __restrict sa, int sarrFree);
/*
 * sarr_array2sarr() - Convert from dynamic array to split-order array
 *
 * @a = array
 * @segLen = Length of segment
 * @arrFree = after convert array !=0 will be destroyed
 * return: NULL error or != NULL new sarr
 */
sarr_t *sarr_array2sarr(array_t ** __restrict a, int segLen, int arrFree);


#endif
