/*************************************************************************
* (C) 2011 AITNET ltd - Sofia/Bulgaria - <misho@aitbg.com>
*  by Michael Pounov <misho@openbsd-bg.org>
*
* $Author: misho $
* $Id: mqttapi.h,v 1.4 2012/07/03 08:57:04 misho Exp $
*
**************************************************************************
The ELWIX and AITNET software is distributed under the following
terms:

All of the documentation and software included in the ELWIX and AITNET
Releases is copyrighted by ELWIX - Sofia/Bulgaria <info@elwix.org>

Copyright 2004, 2005, 2006, 2007, 2008, 2009, 2010, 2011, 2012
	by Michael Pounov <misho@elwix.org>.  All rights reserved.

Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions
are met:
1. Redistributions of source code must retain the above copyright
   notice, this list of conditions and the following disclaimer.
2. Redistributions in binary form must reproduce the above copyright
   notice, this list of conditions and the following disclaimer in the
   documentation and/or other materials provided with the distribution.
3. All advertising materials mentioning features or use of this software
   must display the following acknowledgement:
This product includes software developed by Michael Pounov <misho@elwix.org>
ELWIX - Embedded LightWeight unIX and its contributors.
4. Neither the name of AITNET nor the names of its contributors
   may be used to endorse or promote products derived from this software
   without specific prior written permission.

THIS SOFTWARE IS PROVIDED BY AITNET AND CONTRIBUTORS ``AS IS'' AND
ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
ARE DISCLAIMED.  IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE LIABLE
FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
SUCH DAMAGE.
*/
#ifndef __MQTTAPI_H
#define __MQTTAPI_H


#include "aitmqtt.h"


#ifdef API_SERVER_SIDE
/* MQTT server structure & hooks */

/* MQTT command callback. 1st argument is (mqtt_srv_t*) server handle, 2nd is ready length */
typedef int (*mqtt_cb_t)(void* /*mqtt_srv_t**/, int, void *);
typedef struct {
	mqtt_cb_t	cmds[MQTT_TYPE_MAX];

	int		sock;
	unsigned short	timeout;
	mqtt_msg_t	*buf;
} mqtt_srv_t;
#define mqtt_srv_setCmd(x, _cmd, _func)	(assert((x)), (x)->cmds[(_cmd)] = (_func))
#define mqtt_srv_getCmd(x, _cmd)	(assert((x)), (x)->cmds[(_cmd)])


/*
 * mqtt_srv_cliInit() Init MQTT server side support for clients
 *
 * @sock = Client socket from accept()
 * @buf = Message buffer
 * @timeout = timeout
 * @nb = Non block socket
 * return: NULL error or !=NULL allocated server handle
 */
mqtt_srv_t *mqtt_srv_cliInit(int sock, mqtt_msg_t * __restrict buf, 
		unsigned short timeout, int nb);
/*
 * mqtt_srv_Fini() Finish MQTT server side support for clients
 *
 * @psrv = Server handle
 * return: none
 */
void mqtt_srv_cliFini(mqtt_srv_t ** __restrict psrv);
/*
 * mqtt_srv_Dispatch() MQTT server dispatcher for clients
 *
 * @srv = Server handle
 * @rlen = Ready bytes to parse
 * @arg = Argument pass to command callback
 * return: -1 error or 0 ok
 */
int mqtt_srv_cliDispatch(mqtt_srv_t * __restrict srv, int rlen, void *arg);

/*
 * mqtt_srv_Create() - Create server socket
 *
 * @sa = Server bind address
 * @salen = Server struct sockaddr size
 * return: -1 error or >-1 server socket
 */
int mqtt_srv_Create(struct sockaddr * __restrict sa, int salen);
/*
 * mqtt_srv_Destroy() - Close server socket
 *
 * @sock = Server socket
 * return: -1 error or 0 ok
 */
int mqtt_srv_Destroy(int sock);
/*
 * mqtt_srv_Listen() - Listen server socket
 *
 * @sock = Server socket
 * @maxconn = max number of pending connections
 * @nb = Non block socket
 * return: -1 error or 0 ok
 */
int mqtt_srv_Listen(int sock, int maxconn, int nb);
#endif

/*
 * mqtt_KeepAlive() - Keep Alive check routine
 *
 * @sock = connected socket
 * @ka = keep alive timeout
 * @tries = tries for receive correct ping response, usually ==1
 * return: -1 error, 0 host is alive, 1 timeout session or 2 broken session
 */
int mqtt_KeepAlive(int sock, unsigned short ka, unsigned char tries);
/*
 * mqtt_WillMessage() - Publish WILL message
 *
 * @sock = connected socket
 * @ka = keep alive timeout
 * @topic = will topic
 * @data = will message
 * return: -1 error, 1 timeout, 2 not ack or 0 ok
 */
int mqtt_WillMessage(int sock, unsigned short ka, const char *topic, const char *data);


#ifdef API_CLIENT_SIDE
/* MQTT client structure */

typedef struct {
	int		sock;
	unsigned short	timeout;
	mqtt_msg_t	*buf;
} mqtt_cli_t;


/*
 * mqtt_cli_Open() - Open client connection to MQTT broker
 *
 * @addr = brokers address
 * @timeout = timeout
 * return: NULL error or !=NULL connected to broker
 */
mqtt_cli_t *mqtt_cli_Open(struct sockaddr *addr, unsigned short timeout);
/*
 * mqtt_cli_Close() - Close client connection
 *
 * @cli = connected client
 * return: -1 error or 0 disconnected client and freed all resources
 */
int mqtt_cli_Close(mqtt_cli_t ** __restrict cli);

/*
 * mqtt_cli_Subscribe() - Subscribe to broker
 *
 * @cli = connected client
 * @Topics = Topics for subscribes
 * @msgID = Message ID
 * @Dup = Duplicated request
 * @QoS = Message QoS
 * return: NULL error or !=NULL allocated array with subscribed QoS responses, 
 * 	must be free() result!
 */
unsigned char *mqtt_cli_Subscribe(mqtt_cli_t * __restrict cli, 
		mqtt_subscr_t * __restrict Topics, unsigned short msgID, 
		unsigned char Dup, unsigned char QoS);
/*
 * mqtt_cli_Unsubscribe() - Unsubscribe from broker
 *
 * @cli = connected client
 * @Topics = Topics for unsubscribes
 * @msgID = Message ID
 * @Dup = Duplicated request
 * @QoS = Message QoS
 * return: -1 error or 0 ok
 */
int mqtt_cli_Unsubscribe(mqtt_cli_t * __restrict cli, 
		mqtt_subscr_t * __restrict Topics, unsigned short msgID, 
		unsigned char Dup, unsigned char QoS);
/*
 * mqtt_cli_Publish() - Publish message to broker
 *
 * @cli = connected client
 * @msgID = Message ID
 * @Dup = Duplicated request
 * @QoS = Message QoS
 * @Retain = Retain message
 * @csTopic = Topic
 * @pData = Data
 * @datLen = Data length
 * return: -1 error or > -1 sended bytes
 */
int mqtt_cli_Publish(mqtt_cli_t * __restrict cli, unsigned short msgID, 
		unsigned char Dup, unsigned char QoS, unsigned char Retain, 
		const char *csTopic, const void *pData, int datLen);
#endif


#endif
